
library ieee;
use ieee.std_logic_1164.all;
--use ieee.numeric_std.all;

--
-- performs the operation b <= shift_right (a, sel)
--
entity BarrelShifter is
   --generic (width : positive := 48);
   port (a : in std_logic_vector (47 downto 0);
         b : out std_logic_vector (47 downto 0);
         sel : in std_logic_vector (2 downto 0));
end entity BarrelShifter;


architecture RTL of BarrelShifter is
   signal lsb : std_logic_vector (47 downto 0);
   signal sel1 : std_logic_vector (1 downto 0);
   signal sel2 : std_logic;

   subtype data_type is std_logic_vector (47 downto 0);
   function shift_right (a : in data_type; n : in natural) return data_type is
   begin
      return To_StdLogicVector (To_bitvector(a) sra n);
   end;

begin
   -- split sel into LSB and MSB
   sel1 <= sel (1 downto 0);
   sel2 <= sel (2);
   -- shift by 0, 1, 2 or 4
   LSBShifter: process (a, sel1)
   begin
      case sel1 is
         when B"00" =>
            lsb <= a;
         when B"01" =>
            lsb <= shift_right (a, 1);
         when B"10" =>
            lsb <= shift_right (a, 2);
         when B"11" =>
            lsb <= shift_right (a, 3);
         when others =>
            lsb <= a;
      end case;
   end process LSBShifter;
   -- shift by 0 or 4
   MSBShifter: process (lsb, sel2)
   begin
      case sel2 is
         when '0' =>
            b <= lsb;
         when '1' =>
            b <= shift_right (lsb, 4);
         when others =>
            b <= lsb;
      end case;
   end process MSBShifter;
end architecture RTL;

