
library ieee;
use ieee.std_logic_1164.all;

--
-- performs the operation b <= shift_right (a, sel)
--
entity BarrelShifter3 is
   --generic (width : positive := 48);
   port (a : in std_logic_vector (47 downto 0);
         b : out std_logic_vector (47 downto 0);
         sel : in std_logic_vector (2 downto 0));
end entity BarrelShifter3;


architecture RTL of BarrelShifter3 is

   component Barrel is
      port (a : in std_logic_vector (6 downto 0);
            cin : in std_logic_vector (3 downto 0);
            cout : out std_logic_vector (3 downto 0);
            b : out std_logic_vector (3 downto 0);
            sel : in std_logic_vector (2 downto 0));
   end component Barrel;

   signal aext : std_logic_vector (47+3 downto 0);
   signal carry : std_logic_vector (47+4 downto 0);
begin
   -- sign extend a and carry
   aext (a'left downto a'right) <= a;
   aext (a'left+1) <= a(a'left);
   aext (a'left+2) <= a(a'left);
   aext (a'left+3) <= a(a'left);
   carry (a'left+1) <= a(a'left);
   carry (a'left+2) <= a(a'left);
   carry (a'left+3) <= a(a'left);
   carry (a'left+4) <= a(a'left);
   barrel_array: for i in 11 downto 0 generate
   begin
      barrel_cell: component Barrel
         port map (a => aext(4*i+6 downto 4*i),
                   cin => carry(4*(i+1)+3 downto 4*(i+1)),
                   cout => carry(4*i+3 downto 4*i),
                   b => b(4*i+3 downto 4*i),
                   sel =>sel);
   end generate barrel_array;
end architecture RTL;

